unit JTAGDLLFuncUnit;

interface

uses
  Windows, Classes, SysUtils, DateUtils, Dialogs, Controls, FT2232CJTAGUnit;

type FtcOpenedDevice = record
  ftHandle: Dword;
  DeviceName: String;
end;

type FtcLocationID = record
  LocationID: Dword;
  DeviceName: String;
end;

type TBitLengths = array[0..1, 0..999] of integer;

type FtcBitLengths = record
  BitLengths: TBitLengths;
  NumBitLengths: integer;
end;

procedure DisplayFT2232CJTAGDllError(ProgFunctionName, DllFunctionName: String; ftStatus: FTC_STATUS);
function  GetJTAGDeviceNames(var DeviceNames: TStringList): Boolean;
function  OpenJTAGDevice(DeviceName: String; ClockDivisor: Dword): Boolean;
procedure CloseJTAGDevice(DeviceName: String; DeviceItemIndex: Dword);
function  JTAGScanIn(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToRead: Dword; pReadDataBuffer: PReadDataByteBuffer; pdwNumBytesReturned: PDword): Boolean;
function  JTAGScanOut(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToWrite: Dword; pWriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword): Boolean;
function  JTAGScanInOut(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; pWriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; pReadDataBuffer: PReadDataByteBuffer; pdwNumBytesReturned: PDword): Boolean;
procedure JTAGClearDeviceCmdSequence;
function  HexByteToStr(Dval : integer) : string;
function  JTAGExecuteDeviceCommandSequence(DeviceHandleIndex: Dword; var DisplayLines: TStringList): Boolean;

var
  FT2232CJTAG: TFT2232CJTAG;
  FT2232CJTAGDll: Boolean;
  ftHandle: Dword;
  NumOpenedDevices: Dword;
  ftHandles : array[0..49] of FtcOpenedDevice;
  LocationIDs : array[0..49] of FtcLocationID;
  //BitLengths: array[0..1, 0..999] of integer;
  //NumBitLengths: integer;
  DeviceBitLengths : Array [0 .. 49] of FtcBitLengths;

const ScanInCmd = 0;
const ScanInOutCmd = 1;

implementation

procedure DisplayFT2232CJTAGDllError(ProgFunctionName, DllFunctionName: String; ftStatus: FTC_STATUS);
var
  ErrorMsg: TErrorMessage;
begin
  FT2232CJTAG.GetErrorCodeString(ENGLISH, ftStatus, @ErrorMsg, MAX_NUM_ERROR_MESSAGE_CHARS);

  ShowMessage('Program function name = ' + ProgFunctionName +
              ', FT2232CJTAG Dll function name = ' + DllFunctionName +
              ', Error message = ' + ErrorMsg);
end;

function GetJTAGDeviceNames(var DeviceNames: TStringList): Boolean;
var
  ftStatus: FTC_STATUS;
  NumDevices, DeviceIndex: Dword;
  DeviceName: TDeviceName;
  LocationID: Dword;
begin
  if (FT2232CJTAGDll = True) then
  begin
    NumDevices := 0;
    ftStatus := FT2232CJTAG.GetNumDevices(@NumDevices);

    if (ftStatus = FTC_SUCCESS) then
    begin
      if (NumDevices > 0) then
      begin
        for DeviceIndex := 0 to 49 do
        begin
          LocationIDs[DeviceIndex].DeviceName := '';
          LocationIDs[DeviceIndex].LocationID := 0;
        end;

        DeviceIndex := 0;
        repeat
          ftStatus := FT2232CJTAG.GetDeviceNameLocID(DeviceIndex, @DeviceName, MAX_NUM_DEVICE_NAME_CHARS, @LocationID);

          DeviceNames.Add(DeviceName);

          LocationIDs[DeviceIndex].DeviceName := DeviceName;
          LocationIDs[DeviceIndex].LocationID := LocationID;

          Inc(DeviceIndex);
        until ((DeviceIndex >= NumDevices) or (ftStatus <> FTC_SUCCESS));

        if (ftStatus <> FTC_SUCCESS) then
          DisplayFT2232CJTAGDllError('GetNames', 'GetDeviceNameLocID', ftStatus);
      end
      else
        ShowMessage('No FT2232C dual types devices connected.');
    end
    else
      DisplayFT2232CJTAGDllError('GetNames', 'GetNumDevices', ftStatus);
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;
end;

function OpenJTAGDevice(DeviceName: String; ClockDivisor: Dword): Boolean;
var
  ftStatus: FTC_STATUS;
  LocationID: Dword;
  DeviceHandleIndex: Dword;
  ClockFrequencyHz: Dword;
  bDeviceStored: Boolean;
begin
  if (FT2232CJTAGDll = True) then
  begin
    for DeviceHandleIndex := 0 to 49 do
    begin
      if (LocationIDs[DeviceHandleIndex].DeviceName = DeviceName) then
        LocationID := LocationIDs[DeviceHandleIndex].LocationID;
    end;

    ftStatus := FT2232CJTAG.OpenEx(DeviceName, LocationID, @ftHandle);

    if (ftStatus = FTC_SUCCESS) then
    begin
      ftStatus := FT2232CJTAG.InitDevice(ftHandle, ClockDivisor);

      if (ftStatus = FTC_SUCCESS) then
      begin
        ftStatus := FT2232CJTAG.GetClock(ClockDivisor, @ClockFrequencyHz);

        if (ftStatus = FTC_SUCCESS) then
        begin
          bDeviceStored := False;

          for DeviceHandleIndex := 0 to 49 do
          begin
            if ((ftHandles[DeviceHandleIndex].DeviceName = '') and
                (ftHandles[DeviceHandleIndex].ftHandle = 0) and (not bDeviceStored)) then
            begin
              ftHandles[DeviceHandleIndex].ftHandle := ftHandle;
              ftHandles[DeviceHandleIndex].DeviceName := DeviceName;
              bDeviceStored := True;
            end;
          end;

          Inc(NumOpenedDevices);
        end
        else
          DisplayFT2232CJTAGDllError('OpenDevice', 'GetClock', ftStatus);
      end
      else
        DisplayFT2232CJTAGDllError('OpenDevice', 'InitDevice', ftStatus);

      if (ftStatus <> FTC_SUCCESS) then
      begin
        FT2232CJTAG.Close(ftHandle);

        ftHandle := 0;
      end;
    end
    else
      DisplayFT2232CJTAGDllError('OpenDevice', 'OpenEx', ftStatus);
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;
end;

procedure CloseJTAGDevice(DeviceName: String; DeviceItemIndex: Dword);
var
  DeviceHandleIndex: Dword;
begin
  if (FT2232CJTAGDll = True) then
  begin
    if (fthandle <> 0) then
    begin
      FT2232CJTAG.Close(fthandle);

      if (NumOpenedDevices > 1) then
      begin
        if (DeviceItemIndex <> -1) then
        begin
          fthandle := 0;

          for DeviceHandleIndex := 0 to 49 do
          begin
            if (ftHandles[DeviceHandleIndex].DeviceName = DeviceName) then
            begin
              ftHandles[DeviceHandleIndex].ftHandle := ftHandle;
              ftHandles[DeviceHandleIndex].DeviceName := '';
            end;
          end;
        end;
      end
      else
      begin
        fthandle := 0;

        if (DeviceItemIndex <> -1) then
        begin
          for DeviceHandleIndex := 0 to 49 do
          begin
            if (ftHandles[DeviceHandleIndex].DeviceName = DeviceName) then
            begin
              ftHandles[DeviceHandleIndex].ftHandle := ftHandle;
              ftHandles[DeviceHandleIndex].DeviceName := '';
            end;
          end;
        end;
      end;

      if (NumOpenedDevices > 0) then
        Dec(NumOpenedDevices);
    end;
  end;
end;

function JTAGScanIn(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToRead: Dword; pReadDataBuffer: PReadDataByteBuffer; pdwNumBytesReturned: PDword): Boolean;
var
  ftStatus: FTC_STATUS;
  DeviceHandleIndex, DeviceIndex: Dword;
begin
  if (FT2232CJTAGDll = True) then
  begin
    if bBufferScanCommands then
    begin
      ftStatus := FT2232CJTAG.AddDeviceReadCmd(fthandle, bInstructionTestData, NumBitsToRead, RUN_TEST_IDLE_STATE);

      if (ftStatus = FTC_SUCCESS) then
      begin
        DeviceIndex := 0;
        for DeviceHandleIndex := 0 to 49 do
        begin
          if (ftHandles[DeviceHandleIndex].fthandle = fthandle) then
            DeviceIndex := DeviceHandleIndex;
        end;

        DeviceBitLengths[DeviceIndex].BitLengths[0][DeviceBitLengths[DeviceIndex].NumBitLengths] := NumBitsToRead;
        DeviceBitLengths[DeviceIndex].BitLengths[1][DeviceBitLengths[DeviceIndex].NumBitLengths] := ScanInCmd;
        Inc(DeviceBitLengths[DeviceIndex].NumBitLengths);
      end
      else
        DisplayFT2232CJTAGDllError('JTAGScanIn', 'AddDeviceReadCmd', ftStatus);
    end
    else
    begin
      ftStatus := FT2232CJTAG.Read(ftHandle, bInstructionTestData, NumBitsToRead,
                                   pReadDataBuffer, pdwNumBytesReturned, RUN_TEST_IDLE_STATE);

      if (ftStatus <> FTC_SUCCESS) then
        DisplayFT2232CJTAGDllError('JTAGScanIn', 'Read', ftStatus);
    end;
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;  
end;

function JTAGScanOut(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToWrite: Dword; pWriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword): Boolean;
var
  ftStatus: FTC_STATUS;
begin
  if (FT2232CJTAGDll = True) then
  begin
    if bBufferScanCommands then
    begin
      ftStatus := FT2232CJTAG.AddDeviceWriteCmd(fthandle, bInstructionTestData, NumBitsToWrite, pWriteDataBuffer, NumBytesToWrite, RUN_TEST_IDLE_STATE);

      if (ftStatus <> FTC_SUCCESS) then
        DisplayFT2232CJTAGDllError('JTAGScanOut', 'AddDeviceWriteCmd', ftStatus);
    end
    else
    begin
      ftStatus := FT2232CJTAG.Write(ftHandle, bInstructionTestData, NumBitsToWrite,
                                    pWriteDataBuffer, NumBytesToWrite, RUN_TEST_IDLE_STATE);

     if (ftStatus <> FTC_SUCCESS) then
       DisplayFT2232CJTAGDllError('JTAGScanOut', 'Write', ftStatus);
     end;
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;    
end;

function JTAGScanInOut(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; pWriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; pReadDataBuffer: PReadDataByteBuffer; pdwNumBytesReturned: PDword): Boolean;
var
  ftStatus: FTC_STATUS;
  DeviceHandleIndex, DeviceIndex: Dword;
begin
  if (FT2232CJTAGDll = True) then
  begin
    if bBufferScanCommands then
    begin
      ftStatus := FT2232CJTAG.AddDeviceWriteReadCmd(fthandle, bInstructionTestData, NumBitsToWriteRead, pWriteDataBuffer, NumBytesToWrite, RUN_TEST_IDLE_STATE);

      if (ftStatus = FTC_SUCCESS) then
      begin
        DeviceIndex := 0;
        for DeviceHandleIndex := 0 to 49 do
        begin
          if (ftHandles[DeviceHandleIndex].fthandle = fthandle) then
            DeviceIndex := DeviceHandleIndex;
        end;

        DeviceBitLengths[DeviceIndex].BitLengths[0][DeviceBitLengths[DeviceIndex].NumBitLengths] := NumBitsToWriteRead;
        DeviceBitLengths[DeviceIndex].BitLengths[1][DeviceBitLengths[DeviceIndex].NumBitLengths] := ScanInOutCmd;

        Inc(DeviceBitLengths[DeviceIndex].NumBitLengths);
      end
      else
        DisplayFT2232CJTAGDllError('JTAGScanInOut', 'AddDeviceWriteReadCmd', ftStatus);
    end
    else
    begin
      ftStatus := FT2232CJTAG.WriteRead(ftHandle, bInstructionTestData,
                                        NumBitsToWriteRead, pWriteDataBuffer,
                                        NumBytesToWrite, pReadDataBuffer,
                                        pdwNumBytesReturned, RUN_TEST_IDLE_STATE);

      if (ftStatus <> FTC_SUCCESS) then
        DisplayFT2232CJTAGDllError('JTAGScanInOut', 'WriteRead', ftStatus);
    end;
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;
end;

procedure JTAGClearDeviceCmdSequence;
var
  ftStatus: FTC_STATUS;
  DeviceHandleIndex, DeviceIndex: Dword;
begin
  if (FT2232CJTAGDll = True) then
  begin
    DeviceIndex := 0;
    for DeviceHandleIndex := 0 to 49 do
    begin
      if (ftHandles[DeviceHandleIndex].fthandle = fthandle) then
        DeviceIndex := DeviceHandleIndex;
    end;
    
    DeviceBitLengths[DeviceIndex].NumBitLengths := 0;

    ftStatus := FT2232CJTAG.ClearDeviceCmdSequence(fthandle);

    if (ftStatus <> FTC_SUCCESS) then
      DisplayFT2232CJTAGDllError('JTAGClearDeviceCmdSequence', 'ClearDeviceCmdSequence', ftStatus);
  end;
end;

function HexByteToStr(Dval : integer) : string;
var i : integer;
retstr : string;
begin
retstr := '';
i := (Dval AND $F0) DIV $10;
case i of
  0 : retstr := retstr + '0';
  1 : retstr := retstr + '1';
  2 : retstr := retstr + '2';
  3 : retstr := retstr + '3';
  4 : retstr := retstr + '4';
  5 : retstr := retstr + '5';
  6 : retstr := retstr + '6';
  7 : retstr := retstr + '7';
  8 : retstr := retstr + '8';
  9 : retstr := retstr + '9';
  10 : retstr := retstr + 'A';
  11 : retstr := retstr + 'B';
  12 : retstr := retstr + 'C';
  13 : retstr := retstr + 'D';
  14 : retstr := retstr + 'E';
  15 : retstr := retstr + 'F';
end;
i := Dval AND $F;
case i of
  0 : retstr := retstr + '0';
  1 : retstr := retstr + '1';
  2 : retstr := retstr + '2';
  3 : retstr := retstr + '3';
  4 : retstr := retstr + '4';
  5 : retstr := retstr + '5';
  6 : retstr := retstr + '6';
  7 : retstr := retstr + '7';
  8 : retstr := retstr + '8';
  9 : retstr := retstr + '9';
  10 : retstr := retstr + 'A';
  11 : retstr := retstr + 'B';
  12 : retstr := retstr + 'C';
  13 : retstr := retstr + 'D';
  14 : retstr := retstr + 'E';
  15 : retstr := retstr + 'F';
end;
HexByteToStr := retstr;
end;

function JTAGExecuteDeviceCommandSequence(DeviceHandleIndex: Dword; var DisplayLines: TStringList): Boolean;
var
  ftStatus: FTC_STATUS;
  ExecftHandle: Dword;
  ReadDataBuffer: ReadDataByteBuffer;
  NumBytesReturned, NumDataBytesUsed: Dword;
  BitLengthsIndex, BitLength, NumBytes, ByteIndex: integer;
  ScanCmd: integer;
  OpenedDeviceName, DataMsg: String;
begin
  if (FT2232CJTAGDll = True) then
  begin
    ExecftHandle := ftHandles[DeviceHandleIndex].ftHandle;

    OpenedDeviceName := ftHandles[DeviceHandleIndex].DeviceName;

    ftStatus := FT2232CJTAG.ExecuteCmdSequence(ExecftHandle, @ReadDataBuffer, @NumBytesReturned);

    if (ftStatus = FTC_SUCCESS) then
    begin
      if (NumBytesReturned > 0) then
      begin
        NumDataBytesUsed := 0;

        for BitLengthsIndex := 0 to (DeviceBitLengths[DeviceHandleIndex].NumBitLengths - 1) do
        begin
          BitLength := DeviceBitLengths[DeviceHandleIndex].BitLengths[0][BitLengthsIndex];

          NumBytes := (BitLength div 8);

          if ((BitLength mod 8) > 0) then
            Inc(NumBytes);

          ScanCmd := DeviceBitLengths[DeviceHandleIndex].BitLengths[1][BitLengthsIndex];
          if (ScanCmd = ScanInCmd) then
            DataMsg := 'Scan In Cmd '
          else
            DataMsg := 'Scan In And Out Cmd ';

          DataMsg := OpenedDeviceName + ' - ' + DataMsg + 'Read back : ';

          for ByteIndex := (NumBytes - 1) downto 0 do
            DataMsg := DataMsg + HexByteToStr(ReadDataBuffer[(NumDataBytesUsed + ByteIndex)])  +' ';

          DisplayLines.Add(DataMsg);

          NumDataBytesUsed := (NumDataBytesUsed + NumBytes);
        end;
      end;
    end
    else
    begin
      if (ftStatus = FTC_NO_COMMAND_SEQUENCE) then
        DisplayLines.Add(OpenedDeviceName + ' - ' + 'No command sequence found.')
      else
        DisplayFT2232CJTAGDllError('JTAGExecuteDeviceCommandSequence', 'ExecuteCmdSequence', ftStatus);
    end;

    DeviceBitLengths[DeviceHandleIndex].NumBitLengths := 0;
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;
end;

end.
